#include <stdio.h>
#include <sys/file.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

static void
prompt(char *message)
{
    char buffer[1];

    setvbuf(stdout, NULL, _IONBF, 0);
    printf("PID %u: %s ", getpid(), message);
    read(0, buffer, sizeof buffer);
}

static int
open_lock_file(char *path)
{
    int fd;

    prompt("hit return to open lock file!");
    fd = open(path, O_CREAT | O_RDWR, 0644);
    if (fd < 0) {
        fprintf(stderr, "error opening '%s': %s\n", path, strerror(errno));
        exit(1);
    }

    return fd;
}

static void
lock_file(int fd)
{
    prompt("hit return to lock file!");
    if (flock(fd, LOCK_EX | LOCK_NB) < 0) {
        fprintf(stderr, "cannot get exclusive lock: %s\n", strerror(errno));
        exit(1);
    }
}

static void
work(void)
{
    prompt("working, hit return to finish!");
    printf("PID %u: finished work.\n", getpid());
}

static void
unlock_file(int fd)
{
    prompt("hit return to unlock file!");
    if (flock(fd, LOCK_UN) < 0) {
        fprintf(stderr, "cannot unlock: %s\n", strerror(errno));
        exit(1);
    }
}

static void
unlink_file(char *path)
{
    prompt("hit return to unlink file!");
    if (unlink(path) < 0) {
        fprintf(stderr, "warning: unlinking '%s' failed: %s\n",
                        path, strerror(errno));
    }
}

int
main (int argc, char *argv[])
{
    char *pidfile;
    int fd;

    if (argc < 2) {
        fprintf(stderr, "usage: %s PIDFILE\n", argv[0]);
        return 1;
    }

    pidfile = argv[1];

    fd = open_lock_file(pidfile);
    lock_file(fd);
    work();

    /* Feel free to swap the next two steps.  */
    unlock_file(fd);
    unlink_file(pidfile);

    return 0;
}
